@@ -42,6 +42,10 @@
 #include <arpa/inet.h>
 #include "pipebackend.hh"
 
+// The following requirement guarantees UnknownDomainID will get output as "-1"
+// for compatibility.
+static_assert(std::is_signed<domainid_t>::value);
+
 static const char* kBackendId = "[PIPEBackend]";
 
 CoWrapper::CoWrapper(const string& command, int timeout, int abiVersion)
@@ -152,7 +156,7 @@ void PipeBackend::cleanup()
   d_abiVersion = 0;
 }
 
-void PipeBackend::lookup(const QType& qtype, const DNSName& qname, int zoneId, DNSPacket* pkt_p)
+void PipeBackend::lookup(const QType& qtype, const DNSName& qname, domainid_t zoneId, DNSPacket* pkt_p)
 {
   try {
     launch();
@@ -195,7 +199,7 @@ void PipeBackend::lookup(const QType& qtype, const DNSName& qname, int zoneId, D
   d_qname = qname;
 }
 
-bool PipeBackend::list(const DNSName& target, int inZoneId, bool /* include_disabled */)
+bool PipeBackend::list(const ZoneName& target, domainid_t domain_id, bool /* include_disabled */)
 {
   try {
     launch();
@@ -205,16 +209,16 @@ bool PipeBackend::list(const DNSName& target, int inZoneId, bool /* include_disa
 
     // type    qname           qclass  qtype   id      ip-address
     if (d_abiVersion >= 4)
-      query << "AXFR\t" << inZoneId << "\t" << target.toStringRootDot();
+      query << "AXFR\t" << domain_id << "\t" << target.toStringRootDot();
     else
-      query << "AXFR\t" << inZoneId;
+      query << "AXFR\t" << domain_id;
 
     d_coproc->send(query.str());
   }
   catch (PDNSException& ae) {
     g_log << Logger::Error << kBackendId << " Error from coprocess: " << ae.reason << endl;
   }
-  d_qname = DNSName(std::to_string(inZoneId)); // why do we store a number here??
+  d_qname = DNSName(std::to_string(domain_id)); // why do we store a number here??
   return true;
 }
 
