@@ -72,9 +72,8 @@ int ZeroMQConnector::send_message(const Json& input)
   auto line = input.dump();
   zmq_msg_t message;
 
-  zmq_msg_init_size(&message, line.size() + 1);
+  zmq_msg_init_size(&message, line.size());
   line.copy(reinterpret_cast<char*>(zmq_msg_data(&message)), line.size());
-  ((char*)zmq_msg_data(&message))[line.size()] = '\0';
 
   try {
     zmq_pollitem_t item;
