@@ -30,7 +30,7 @@ extern StatBag S;
 
 const unsigned int AuthPacketCache::s_mincleaninterval, AuthPacketCache::s_maxcleaninterval;
 
-AuthPacketCache::AuthPacketCache(size_t mapsCount): d_maps(mapsCount), d_lastclean(time(nullptr))
+AuthPacketCache::AuthPacketCache(size_t mapsCount): d_mapscount(mapsCount), d_lastclean(time(nullptr))
 {
   S.declare("packetcache-hit", "Number of hits on the packet cache");
   S.declare("packetcache-miss", "Number of misses on the packet cache");
@@ -41,6 +41,27 @@ AuthPacketCache::AuthPacketCache(size_t mapsCount): d_maps(mapsCount), d_lastcle
   d_statnumhit=S.getPointer("packetcache-hit");
   d_statnummiss=S.getPointer("packetcache-miss");
   d_statnumentries=S.getPointer("packetcache-size");
+
+  // Create the MapCombo for the default view
+  auto cache = d_cache.write_lock();
+  std::string defaultview{};
+  createViewMap(*cache, defaultview);
+}
+
+// Create the vector<MapCombo> for the given view.
+// Assumes there is no existing data for the view. Callers are expected to
+// know what they are doing.
+std::unordered_map<std::string, std::unique_ptr<vector<AuthPacketCache::MapCombo>>>::iterator AuthPacketCache::createViewMap(cache_t& cache, const std::string& view)
+{
+  auto iter = cache.emplace(view, std::make_unique<vector<MapCombo>>(d_mapscount));
+  auto retval = iter.first;
+  auto* map = retval->second.get();
+  // Note that this reserves more than intended, especially if multiple views
+  // are used.
+  for (auto& shard : *map) {
+    shard.reserve(d_maxEntries / map->size());
+  }
+  return retval;
 }
 
 void AuthPacketCache::MapCombo::reserve(size_t numberOfEntries)
@@ -50,30 +71,39 @@ void AuthPacketCache::MapCombo::reserve(size_t numberOfEntries)
 #endif /* BOOST_VERSION >= 105600 */
 }
 
-bool AuthPacketCache::get(DNSPacket& p, DNSPacket& cached)
+bool AuthPacketCache::get(DNSPacket& pkt, DNSPacket& cached, const std::string& view)
 {
-  if(!d_ttl) {
+  if (d_ttl == 0) {
     return false;
   }
 
   cleanupIfNeeded();
 
   static const std::unordered_set<uint16_t> optionsToSkip{ EDNSOptionCode::COOKIE};
-  uint32_t hash = canHashPacket(p.getString(), /* don't skip ECS */optionsToSkip);
-  p.setHash(hash);
+  uint32_t hash = canHashPacket(pkt.getString(), /* don't skip ECS */optionsToSkip);
+  pkt.setHash(hash);
 
   string value;
   bool haveSomething;
   time_t now = time(nullptr);
-  auto& mc = getMap(p.qdomain);
   {
-    auto map = mc.d_map.try_read_lock();
-    if (!map.owns_lock()) {
-      S.inc("deferred-packetcache-lookup");
+    auto cache = d_cache.read_lock();
+    auto iter = cache->find(view);
+    if (iter == cache->end()) {
+      // No data for this view yet.
+      (*d_statnummiss)++;
       return false;
     }
+    auto& mapcombo = getMap(iter->second, pkt.qdomain);
+    {
+      auto map = mapcombo.d_map.try_read_lock();
+      if (!map.owns_lock()) {
+        S.inc("deferred-packetcache-lookup");
+        return false;
+      }
 
-    haveSomething = getEntryLocked(*map, p.getString(), hash, p.qdomain, p.qtype.getCode(), p.d_tcp, now, value);
+      haveSomething = AuthPacketCache::getEntryLocked(*map, pkt.getString(), hash, pkt.qdomain, pkt.qtype.getCode(), pkt.d_tcp, now, value);
+    }
   }
 
   if (!haveSomething) {
@@ -86,9 +116,9 @@ bool AuthPacketCache::get(DNSPacket& p, DNSPacket& cached)
   }
 
   (*d_statnumhit)++;
-  cached.spoofQuestion(p); // for correct case
-  cached.qdomain = p.qdomain;
-  cached.qtype = p.qtype;
+  cached.spoofQuestion(pkt); // for correct case
+  cached.qdomain = pkt.qdomain;
+  cached.qtype = pkt.qtype;
 
   return true;
 }
@@ -99,79 +129,88 @@ bool AuthPacketCache::entryMatches(cmap_t::index<HashTag>::type::iterator& iter,
   return iter->tcp == tcp && iter->qtype == qtype && iter->qname == qname && queryMatches(iter->query, query, qname, skippedEDNSTypes);
 }
 
-void AuthPacketCache::insert(DNSPacket& q, DNSPacket& r, unsigned int maxTTL)
+void AuthPacketCache::insert(DNSPacket& query, DNSPacket& response, unsigned int maxTTL, const std::string& view)
 {
-  if(!d_ttl) {
+  if (d_ttl == 0) {
     return;
   }
 
   cleanupIfNeeded();
 
-  if (ntohs(q.d.qdcount) != 1) {
+  if (ntohs(query.d.qdcount) != 1) {
     return; // do not try to cache packets with multiple questions
   }
 
-  if (q.qclass != QClass::IN) // we only cache the INternet
+  if (query.qclass != QClass::IN) { // we only cache the INternet
     return;
+  }
 
   uint32_t ourttl = std::min(d_ttl, maxTTL);
   if (ourttl == 0) {
     return;
-  }  
+  }
 
-  uint32_t hash = q.getHash();
+  uint32_t hash = query.getHash();
   time_t now = time(nullptr);
   CacheEntry entry;
   entry.hash = hash;
   entry.created = now;
   entry.ttd = now + ourttl;
-  entry.qname = q.qdomain;
-  entry.qtype = q.qtype.getCode();
-  entry.value = r.getString();
-  entry.tcp = r.d_tcp;
-  entry.query = q.getString();
-  
-  auto& mc = getMap(entry.qname);
+  entry.qname = query.qdomain;
+  entry.qtype = query.qtype.getCode();
+  entry.value = response.getString();
+  entry.tcp = response.d_tcp;
+  entry.query = query.getString();
+
   {
-    auto map = mc.d_map.try_write_lock();
-    if (!map.owns_lock()) {
-      S.inc("deferred-packetcache-inserts");
-      return;
+    auto cache = d_cache.write_lock();
+    auto iter = cache->find(view);
+    if (iter == cache->end()) {
+      // No data for this view yet, create it.
+      iter = createViewMap(*cache, view);
     }
-
-    auto& idx = map->get<HashTag>();
-    auto range = idx.equal_range(hash);
-    auto iter = range.first;
-
-    for( ; iter != range.second ; ++iter)  {
-      if (!entryMatches(iter, entry.query, entry.qname, entry.qtype, entry.tcp)) {
-        continue;
+    auto& mc = getMap(iter->second, entry.qname); // NOLINT(readability-identifier-length)
+    {
+      auto map = mc.d_map.try_write_lock();
+      if (!map.owns_lock()) {
+        S.inc("deferred-packetcache-inserts");
+        return;
       }
 
-      moveCacheItemToBack<SequencedTag>(*map, iter);
-      iter->value = entry.value;
-      iter->ttd = now + ourttl;
-      iter->created = now;
-      return;
-    }
+      auto& idx = map->get<HashTag>();
+      auto range = idx.equal_range(hash);
+      auto iter2 = range.first;
 
-    /* no existing entry found to refresh */
-    map->insert(std::move(entry));
+      for( ; iter2 != range.second ; ++iter2)  {
+        if (!entryMatches(iter2, entry.query, entry.qname, entry.qtype, entry.tcp)) {
+          continue;
+        }
 
-    if (*d_statnumentries >= d_maxEntries) {
-      /* remove the least recently inserted or replaced entry */
-      auto& sidx = map->get<SequencedTag>();
-      sidx.pop_front();
-    }
-    else {
-      ++(*d_statnumentries);
+        moveCacheItemToBack<SequencedTag>(*map, iter2);
+        iter2->value = entry.value;
+        iter2->ttd = now + ourttl;
+        iter2->created = now;
+        return;
+      }
+
+      /* no existing entry found to refresh */
+      map->insert(std::move(entry));
+
+      if (*d_statnumentries >= d_maxEntries) {
+        /* remove the least recently inserted or replaced entry */
+        auto& sidx = map->get<SequencedTag>();
+        sidx.pop_front();
+      }
+      else {
+        ++(*d_statnumentries);
+      }
     }
   }
 }
 
 bool AuthPacketCache::getEntryLocked(const cmap_t& map, const std::string& query, uint32_t hash, const DNSName &qname, uint16_t qtype, bool tcp, time_t now, string& value)
 {
-  auto& idx = map.get<HashTag>();
+  const auto& idx = map.get<HashTag>();
   auto range = idx.equal_range(hash);
 
   for(auto iter = range.first; iter != range.second ; ++iter)  {
@@ -182,7 +221,6 @@ bool AuthPacketCache::getEntryLocked(const cmap_t& map, const std::string& query
     if (!entryMatches(iter, query, qname, qtype, tcp)) {
       continue;
     }
-
     value = iter->value;
     return true;
   }
@@ -193,19 +231,52 @@ bool AuthPacketCache::getEntryLocked(const cmap_t& map, const std::string& query
 /* clears the entire cache. */
 uint64_t AuthPacketCache::purge()
 {
-  if(!d_ttl) {
+  if (d_ttl == 0) {
     return 0;
   }
 
   d_statnumentries->store(0);
 
-  return purgeLockedCollectionsVector(d_maps);
+  uint64_t delcount = 0;
+  {
+    auto cache = d_cache.write_lock();
+    for (auto& iter : *cache) {
+      auto* map = iter.second.get();
+      delcount += purgeLockedCollectionsVector(*map);
+    }
+  }
+  return delcount;
 }
 
 uint64_t AuthPacketCache::purgeExact(const DNSName& qname)
 {
-  auto& mc = getMap(qname);
-  uint64_t delcount = purgeExactLockedCollection<NameTag>(mc, qname);
+  uint64_t delcount = 0;
+
+  {
+    auto cache = d_cache.write_lock();
+    for (auto& iter : *cache) {
+      auto& mc = getMap(iter.second, qname); // NOLINT(readability-identifier-length)
+      delcount += purgeExactLockedCollection<NameTag>(mc, qname);
+    }
+  }
+
+  *d_statnumentries -= delcount;
+
+  return delcount;
+}
+
+uint64_t AuthPacketCache::purgeView(const std::string& view)
+{
+  uint64_t delcount = 0;
+
+  {
+    auto cache = d_cache.write_lock();
+    if (auto iter = cache->find(view); iter != cache->end()) {
+      auto* map = iter->second.get();
+      delcount += purgeLockedCollectionsVector(*map);
+      cache->erase(iter);
+    }
+  }
 
   *d_statnumentries -= delcount;
 
@@ -215,14 +286,20 @@ uint64_t AuthPacketCache::purgeExact(const DNSName& qname)
 /* purges entries from the packetcache. If match ends on a $, it is treated as a suffix */
 uint64_t AuthPacketCache::purge(const string &match)
 {
-  if(!d_ttl) {
+  if (d_ttl == 0) {
     return 0;
   }
 
   uint64_t delcount = 0;
 
   if(boost::ends_with(match, "$")) {
-    delcount = purgeLockedCollectionsVector<NameTag>(d_maps, match);
+    {
+      auto cache = d_cache.write_lock();
+      for (auto& iter : *cache) {
+        auto* map = iter.second.get();
+        delcount += purgeLockedCollectionsVector<NameTag>(*map, match);
+      }
+    }
     *d_statnumentries -= delcount;
   }
   else {
@@ -231,10 +308,45 @@ uint64_t AuthPacketCache::purge(const string &match)
 
   return delcount;
 }
-			   
+
+uint64_t AuthPacketCache::purge(const std::string& view, const std::string& match)
+{
+  if (d_ttl == 0) {
+    return 0;
+  }
+
+  uint64_t delcount = 0;
+
+  {
+    auto cache = d_cache.write_lock();
+    if (auto iter = cache->find(view); iter != cache->end()) {
+      if (boost::ends_with(match, "$")) {
+        auto *map = iter->second.get();
+        delcount += purgeLockedCollectionsVector<NameTag>(*map, match);
+      }
+      else {
+        DNSName qname(match);
+        auto& mc = getMap(iter->second, qname); // NOLINT(readability-identifier-length)
+        delcount += purgeExactLockedCollection<NameTag>(mc, qname);
+      }
+    }
+  }
+
+  *d_statnumentries -= delcount;
+
+  return delcount;
+}
+
 void AuthPacketCache::cleanup()
 {
-  uint64_t totErased = pruneLockedCollectionsVector<SequencedTag>(d_maps);
+  uint64_t totErased = 0;
+  {
+    auto cache = d_cache.write_lock();
+    for (auto& iter : *cache) {
+      auto* map = iter.second.get();
+      totErased += pruneLockedCollectionsVector<SequencedTag>(*map);
+    }
+  }
   *d_statnumentries -= totErased;
 
   DLOG(g_log<<"Done with cache clean, cacheSize: "<<(*d_statnumentries)<<", totErased"<<totErased<<endl);
