@@ -23,10 +23,8 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#ifdef HAVE_BOOST_GE_148
-#include "histog.hh"
-#endif
 
+#include "histog.hh"
 #include "statbag.hh"
 #include "dnspcap.hh"
 #include "dnsparser.hh"
@@ -45,6 +43,12 @@
 #include "dnsrecords.hh"
 #include "statnode.hh"
 
+#if !defined(IP_OFFMASK)
+// Solaris and derivatives do not define IP_OFFMASK in <netinet/ip.h>.
+// We can't even use ~(IP_RF | IP_DF | IP_MF) as it doesn't define IP_RF either.
+#define IP_OFFMASK 0x1fff /* mask for fragmenting bits */
+#endif
+
 namespace po = boost::program_options;
 po::variables_map g_vm;
 
@@ -139,10 +143,8 @@ try
     ("rd", po::value<bool>(), "If set to true, only process RD packets, to false only non-RD, unset: both")
     ("ipv4", po::value<bool>()->default_value(true), "Process IPv4 packets")
     ("ipv6", po::value<bool>()->default_value(true), "Process IPv6 packets")
-#ifdef HAVE_BOOST_GE_148
     ("log-histogram", "Write a log-histogram to file 'log-histogram'")
     ("full-histogram", po::value<double>(), "Write a log-histogram to file 'full-histogram' with this millisecond bin size")
-#endif
     ("filter-name,f", po::value<string>(), "Do statistics only for queries within this domain")
     ("load-stats,l", po::value<string>()->default_value(""), "if set, emit per-second load statistics (questions, answers, outstanding)")
     ("no-servfail-stats", "Don't include servfails in response time stats")
@@ -229,7 +231,8 @@ try
   typedef vector<pair<time_t, LiveCounts> > pcounts_t;
   pcounts_t pcounts;
   const uint16_t port = g_vm["port"].as<uint16_t>();
-  OPTRecordContent::report();
+
+  reportAllTypes();
 
   for(unsigned int fno=0; fno < files.size(); ++fno) {
     PcapPacketReader pr(files[fno]);
@@ -472,7 +475,6 @@ try
   cout.precision(4);
   sum=0;
 
-#ifdef HAVE_BOOST_GE_148
   if(g_vm.count("log-histogram")) {
     string fname = g_vm["stats-dir"].as<string>()+"/log-histogram";
     ofstream loglog(fname);
@@ -489,8 +491,6 @@ try
       throw runtime_error("Unable to write statistics to "+fname);
     writeFullHistogramFile(cumul, g_vm["full-histogram"].as<double>(), loglog);
   }
-#endif
-
 
   sum=0;
   double lastperc=0, perc=0;
