@@ -33,16 +33,16 @@
 #include <limits.h>
 
 /* d_content:                                      <---- d_stuff ---->
-                                      v d_truncatemarker  
+                                      v d_truncatemarker
    dnsheader | qname | qtype | qclass | {recordname| dnsrecordheader | record }
-                                        ^ d_rollbackmarker           ^ d_sor 
-    
+                                        ^ d_rollbackmarker           ^ d_sor
+
 
 */
 
-
-template <typename Container> GenericDNSPacketWriter<Container>::GenericDNSPacketWriter(Container& content, const DNSName& qname, uint16_t  qtype, uint16_t qclass, uint8_t opcode)
-  : d_content(content), d_qname(qname), d_canonic(false), d_lowerCase(false)
+template <typename Container>
+GenericDNSPacketWriter<Container>::GenericDNSPacketWriter(Container& content, const DNSName& qname, uint16_t qtype, uint16_t qclass, uint8_t opcode) :
+  d_content(content), d_qname(qname)
 {
   d_content.clear();
   dnsheader dnsheader;
@@ -131,13 +131,13 @@ template <typename Container> void GenericDNSPacketWriter<Container>::addOpt(con
 
 template <typename Container> void GenericDNSPacketWriter<Container>::xfr48BitInt(uint64_t val)
 {
-  unsigned char bytes[6];
+  std::array<unsigned char, 6> bytes;
   uint16_t theLeft = htons((val >> 32)&0xffffU);
   uint32_t theRight = htonl(val & 0xffffffffU);
-  memcpy(bytes, (void*)&theLeft, sizeof(theLeft));
-  memcpy(bytes+2, (void*)&theRight, sizeof(theRight));
+  memcpy(&bytes[0], (void*)&theLeft, sizeof(theLeft));
+  memcpy(&bytes[2], (void*)&theRight, sizeof(theRight));
 
-  d_content.insert(d_content.end(), bytes, bytes + sizeof(bytes));
+  d_content.insert(d_content.end(), bytes.begin(), bytes.end());
 }
 
 template <typename Container> void GenericDNSPacketWriter<Container>::xfrNodeOrLocatorID(const NodeOrLocatorID& val)
@@ -213,15 +213,12 @@ template <typename Container> uint16_t GenericDNSPacketWriter<Container>::lookup
 
   /* name might be a.root-servers.net, we need to be able to benefit from finding:
      b.root-servers.net, or even:
-     b\xc0\x0c 
+     b\xc0\x0c
   */
   unsigned int bestpos=0;
   *matchLen=0;
-#if BOOST_VERSION >= 105400
-  boost::container::static_vector<uint16_t, 34> nvect, pvect;
-#else
-  vector<uint16_t> nvect, pvect;
-#endif
+  boost::container::static_vector<uint16_t, 34> nvect;
+  boost::container::static_vector<uint16_t, 34> pvect;
 
   try {
     for(auto riter= raw.cbegin(); riter < raw.cend(); ) {
@@ -236,10 +233,10 @@ template <typename Container> uint16_t GenericDNSPacketWriter<Container>::lookup
       cout<<"Domain "<<name<<" too large to compress"<<endl;
     return 0;
   }
-  
+
   if(l_verbose) {
     cout<<"Input vector for lookup "<<name<<": ";
-    for(const auto n : nvect) 
+    for(const auto n : nvect)
       cout << n<<" ";
     cout<<endl;
     cout<<makeHexDump(string(raw.c_str(), raw.c_str()+raw.size()))<<endl;
@@ -285,7 +282,7 @@ template <typename Container> uint16_t GenericDNSPacketWriter<Container>::lookup
     }
     if(l_verbose) {
       cout<<"Packet vector: "<<endl;
-      for(const auto n : pvect) 
+      for(const auto n : pvect)
         cout << n<<" ";
       cout<<endl;
     }
@@ -319,7 +316,7 @@ template <typename Container> uint16_t GenericDNSPacketWriter<Container>::lookup
   return bestpos;
 }
 // this is the absolute hottest function in the pdns recursor
-template <typename Container> void GenericDNSPacketWriter<Container>::xfrName(const DNSName& name, bool compress, bool)
+template <typename Container> void GenericDNSPacketWriter<Container>::xfrName(const DNSName& name, bool compress)
 {
   if(l_verbose)
     cout<<"Wants to write "<<name<<", compress="<<compress<<", canonic="<<d_canonic<<", LC="<<d_lowerCase<<endl;
@@ -334,7 +331,7 @@ template <typename Container> void GenericDNSPacketWriter<Container>::xfrName(co
   uint16_t li=0;
   uint16_t matchlen=0;
   if(d_compress && compress && (li=lookupName(name, &matchlen)) && li < maxCompressionOffset) {
-    const auto& dns=name.getStorage(); 
+    const auto& dns=name.getStorage();
     if(l_verbose)
       cout<<"Found a substring of "<<matchlen<<" bytes from the back, offset: "<<li<<", dnslen: "<<dns.size()<<endl;
     // found a substring, if www.powerdns.com matched powerdns.com, we get back matchlen = 13
@@ -392,7 +389,7 @@ template <typename Container> void GenericDNSPacketWriter<Container>::xfrBlobNoS
   xfrBlob(blob);
 }
 
-template <typename Container> void GenericDNSPacketWriter<Container>::xfrHexBlob(const string& blob, bool keepReading)
+template <typename Container> void GenericDNSPacketWriter<Container>::xfrHexBlob(const string& blob, bool /* keepReading */)
 {
   xfrBlob(blob);
 }
@@ -432,13 +429,13 @@ template <typename Container> void GenericDNSPacketWriter<Container>::xfrSvcPara
       break;
     case SvcParam::ipv4hint:
       xfr16BitInt(param.getIPHints().size() * 4); // size
-      for (auto a: param.getIPHints()) {
+      for (const auto& a: param.getIPHints()) {
         xfrCAWithoutPort(param.getKey(), a);
       }
       break;
     case SvcParam::ipv6hint:
       xfr16BitInt(param.getIPHints().size() * 16); // size
-      for (auto a: param.getIPHints()) {
+      for (const auto& a: param.getIPHints()) {
         xfrCAWithoutPort(param.getKey(), a);
       }
       break;
@@ -460,6 +457,12 @@ template <typename Container> void GenericDNSPacketWriter<Container>::getRecordP
   records.assign(d_content.begin() + d_sor, d_content.end());
 }
 
+// call __before commit__
+template <typename Container> void GenericDNSPacketWriter<Container>::getWireFormatContent(string& record)
+{
+  record.assign(d_content.begin() + d_rollbackmarker, d_content.end());
+}
+
 template <typename Container> uint32_t GenericDNSPacketWriter<Container>::size() const
 {
   return d_content.size();
