@@ -60,6 +60,7 @@
 #include <pwd.h>
 #include <grp.h>
 #include <climits>
+#include <unordered_map>
 #ifdef __FreeBSD__
 #  include <pthread_np.h>
 #endif
@@ -95,24 +96,26 @@ size_t writen2(int fileDesc, const void *buf, size_t count)
   return count;
 }
 
-size_t readn2(int fd, void* buffer, size_t len)
+size_t readn2(int fileDesc, void* buffer, size_t len)
 {
-  size_t pos=0;
-  ssize_t res;
-  for(;;) {
-    res = read(fd, (char*)buffer + pos, len - pos);
-    if(res == 0)
+  size_t pos = 0;
+
+  for (;;) {
+    auto res = read(fileDesc, static_cast<char *>(buffer) + pos, len - pos); // NOLINT(cppcoreguidelines-pro-bounds-pointer-arithmetic): it's the API
+    if (res == 0) {
       throw runtime_error("EOF while reading message");
-    if(res < 0) {
-      if (errno == EAGAIN)
+    }
+    if (res < 0) {
+      if (errno == EAGAIN) {
         throw std::runtime_error("used readn2 on non-blocking socket, got EAGAIN");
-      else
-        unixDie("failed in readn2");
+      }
+      unixDie("failed in readn2");
     }
 
-    pos+=(size_t)res;
-    if(pos == len)
+    pos += static_cast<size_t>(res);
+    if (pos == len) {
       break;
+    }
   }
   return len;
 }
@@ -282,7 +285,7 @@ auto pdns::OpenSSL::error(const std::string& errorMessage) -> std::runtime_error
     }
   }
 #endif
-  return std::runtime_error(fullErrorMessage);
+  return std::runtime_error{fullErrorMessage};
 }
 
 auto pdns::OpenSSL::error(const std::string& componentName, const std::string& errorMessage) -> std::runtime_error
@@ -305,69 +308,6 @@ string nowTime()
   return {buffer.data()};
 }
 
-static bool ciEqual(const string& lhs, const string& rhs)
-{
-  if (lhs.size() != rhs.size()) {
-    return false;
-  }
-
-  string::size_type pos = 0;
-  const string::size_type epos = lhs.size();
-  for (; pos < epos; ++pos) {
-    if (dns_tolower(lhs[pos]) != dns_tolower(rhs[pos])) {
-      return false;
-    }
-  }
-  return true;
-}
-
-/** does domain end on suffix? Is smart about "wwwds9a.nl" "ds9a.nl" not matching */
-static bool endsOn(const string &domain, const string &suffix)
-{
-  if( suffix.empty() || ciEqual(domain, suffix) ) {
-    return true;
-  }
-
-  if(domain.size() <= suffix.size()) {
-    return false;
-  }
-
-  string::size_type dpos = domain.size() - suffix.size() - 1;
-  string::size_type spos = 0;
-
-  if (domain[dpos++] != '.') {
-    return false;
-  }
-
-  for(; dpos < domain.size(); ++dpos, ++spos) {
-    if (dns_tolower(domain[dpos]) != dns_tolower(suffix[spos])) {
-      return false;
-    }
-  }
-
-  return true;
-}
-
-/** strips a domain suffix from a domain, returns true if it stripped */
-bool stripDomainSuffix(string *qname, const string &domain)
-{
-  if (!endsOn(*qname, domain)) {
-    return false;
-  }
-
-  if (toLower(*qname) == toLower(domain)) {
-    *qname="@";
-  }
-  else {
-    if ((*qname)[qname->size() - domain.size() - 1] != '.') {
-      return false;
-    }
-
-    qname->resize(qname->size() - domain.size()-1);
-  }
-  return true;
-}
-
 // returns -1 in case if error, 0 if no data is available, 1 if there is. In the first two cases, errno is set
 int waitForData(int fileDesc, int seconds, int useconds)
 {
@@ -566,7 +506,7 @@ std::string getCarbonHostName()
     throw std::runtime_error(stringerror());
   }
 
-  boost::replace_all(*hostname, ".", "_");
+  std::replace(hostname->begin(), hostname->end(), '.', '_');
   return *hostname;
 }
 
@@ -628,15 +568,16 @@ string U32ToIP(uint32_t val)
 }
 
 
-string makeHexDump(const string& str)
+string makeHexDump(const string& str, const string& sep)
 {
   std::array<char, 5> tmp;
   string ret;
-  ret.reserve(static_cast<size_t>(str.size()*2.2));
+  ret.reserve(static_cast<size_t>(str.size() * (2 + sep.size())));
 
   for (char n : str) {
-    snprintf(tmp.data(), tmp.size(), "%02x ", static_cast<unsigned char>(n));
+    snprintf(tmp.data(), tmp.size(), "%02x", static_cast<unsigned char>(n));
     ret += tmp.data();
+    ret += sep;
   }
   return ret;
 }
@@ -727,6 +668,62 @@ int logFacilityToLOG(unsigned int facility)
   }
 }
 
+std::optional<int> logFacilityFromString(std::string facilityStr)
+{
+  static std::unordered_map<std::string, int> const s_facilities = {
+    {"local0", LOG_LOCAL0},
+    {"log_local0", LOG_LOCAL0},
+    {"local1", LOG_LOCAL1},
+    {"log_local1", LOG_LOCAL1},
+    {"local2", LOG_LOCAL2},
+    {"log_local2", LOG_LOCAL2},
+    {"local3", LOG_LOCAL3},
+    {"log_local3", LOG_LOCAL3},
+    {"local4", LOG_LOCAL4},
+    {"log_local4", LOG_LOCAL4},
+    {"local5", LOG_LOCAL5},
+    {"log_local5", LOG_LOCAL5},
+    {"local6", LOG_LOCAL6},
+    {"log_local6", LOG_LOCAL6},
+    {"local7", LOG_LOCAL7},
+    {"log_local7", LOG_LOCAL7},
+    /* most of these likely make very little sense
+       for us, but why not? */
+    {"kern", LOG_KERN},
+    {"log_kern", LOG_KERN},
+    {"user", LOG_USER},
+    {"log_user", LOG_USER},
+    {"mail", LOG_MAIL},
+    {"log_mail", LOG_MAIL},
+    {"daemon", LOG_DAEMON},
+    {"log_daemon", LOG_DAEMON},
+    {"auth", LOG_AUTH},
+    {"log_auth", LOG_AUTH},
+    {"syslog", LOG_SYSLOG},
+    {"log_syslog", LOG_SYSLOG},
+    {"lpr", LOG_LPR},
+    {"log_lpr", LOG_LPR},
+    {"news", LOG_NEWS},
+    {"log_news", LOG_NEWS},
+    {"uucp", LOG_UUCP},
+    {"log_uucp", LOG_UUCP},
+    {"cron", LOG_CRON},
+    {"log_cron", LOG_CRON},
+    {"authpriv", LOG_AUTHPRIV},
+    {"log_authpriv", LOG_AUTHPRIV},
+    {"ftp", LOG_FTP},
+    {"log_ftp", LOG_FTP}
+  };
+
+  toLowerInPlace(facilityStr);
+  auto facilityIt = s_facilities.find(facilityStr);
+  if (facilityIt == s_facilities.end()) {
+    return std::nullopt;
+  }
+
+  return facilityIt->second;
+}
+
 string stripDot(const string& dom)
 {
   if(dom.empty())
@@ -861,11 +858,11 @@ bool stringfgets(FILE* fp, std::string& line)
 bool readFileIfThere(const char* fname, std::string* line)
 {
   line->clear();
-  auto fp = std::unique_ptr<FILE, int(*)(FILE*)>(fopen(fname, "r"), fclose);
-  if (!fp) {
+  auto filePtr = pdns::UniqueFilePtr(fopen(fname, "r"));
+  if (!filePtr) {
     return false;
   }
-  return stringfgets(fp.get(), *line);
+  return stringfgets(filePtr.get(), *line);
 }
 
 Regex::Regex(const string &expr)
@@ -1022,6 +1019,36 @@ bool setReuseAddr(int sock)
   return true;
 }
 
+void setDscp(int sock, unsigned short family, uint8_t dscp)
+{
+  int val = 0;
+  unsigned int len = 0;
+
+  if (dscp == 0 || dscp > 63) {
+    // No DSCP marking
+    return;
+  }
+
+  if (family == AF_INET) {
+    if (getsockopt(sock, IPPROTO_IP, IP_TOS, &val, &len)<0) {
+      throw std::runtime_error(string("Set DSCP failed: ")+stringerror());
+    }
+    val = (dscp<<2) | (val&0x3);
+    if (setsockopt(sock, IPPROTO_IP, IP_TOS, &val, sizeof(val))<0) {
+      throw std::runtime_error(string("Set DSCP failed: ")+stringerror());
+    }
+  }
+  else if (family == AF_INET6) {
+    if (getsockopt(sock, IPPROTO_IPV6, IPV6_TCLASS, &val, &len)<0) {
+      throw std::runtime_error(string("Set DSCP failed: ")+stringerror());
+    }
+    val = (dscp<<2) | (val&0x3);
+    if (setsockopt(sock, IPPROTO_IPV6, IPV6_TCLASS, &val, sizeof(val))<0) {
+      throw std::runtime_error(string("Set DSCP failed: ")+stringerror());
+    }
+  }
+}
+
 bool isNonBlocking(int sock)
 {
   int flags=fcntl(sock,F_GETFL,0);
@@ -1751,9 +1778,16 @@ bool constantTimeStringEquals(const std::string& a, const std::string& b)
 
 namespace pdns
 {
+struct CloseDirDeleter
+{
+  void operator()(DIR* dir) const noexcept {
+    closedir(dir);
+  }
+};
+
 std::optional<std::string> visit_directory(const std::string& directory, const std::function<bool(ino_t inodeNumber, const std::string_view& name)>& visitor)
 {
-  auto dirHandle = std::unique_ptr<DIR, decltype(&closedir)>(opendir(directory.c_str()), closedir);
+  auto dirHandle = std::unique_ptr<DIR, CloseDirDeleter>(opendir(directory.c_str()));
   if (!dirHandle) {
     auto err = errno;
     return std::string("Error opening directory '" + directory + "': " + stringerror(err));
@@ -1770,4 +1804,28 @@ std::optional<std::string> visit_directory(const std::string& directory, const s
 
   return std::nullopt;
 }
+
+UniqueFilePtr openFileForWriting(const std::string& filePath, mode_t permissions, bool mustNotExist, bool appendIfExists)
+{
+  int flags = O_WRONLY | O_CREAT;
+  if (mustNotExist) {
+    flags |= O_EXCL;
+  }
+  else if (appendIfExists) {
+    flags |= O_APPEND;
+  }
+  int fileDesc = open(filePath.c_str(), flags, permissions);
+  if (fileDesc == -1) {
+    return {};
+  }
+  auto filePtr = pdns::UniqueFilePtr(fdopen(fileDesc, appendIfExists ? "a" : "w"));
+  if (!filePtr) {
+    auto error = errno;
+    close(fileDesc);
+    errno = error;
+    return {};
+  }
+  return filePtr;
+}
+
 }
