@@ -19,14 +19,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+
+#include "config.h"
 #include "misc.hh"
 #include <memory>
 #include <openssl/crypto.h>
 #include <openssl/ec.h>
 #include <optional>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include <openssl/obj_mac.h>
 #ifdef HAVE_LIBCRYPTO_ECDSA
 #include <openssl/ecdsa.h>
@@ -942,7 +941,7 @@ void OpenSSLRSADNSCryptoKeyEngine::fromPublicKeyString(const std::string& conten
     if (contentLen < 3) {
       throw runtime_error(getName() + " invalid input size for the public key");
     }
-    const size_t exponentSize = (static_cast<size_t>(raw[1])) * 0x100 + raw[2];
+    const size_t exponentSize = raw[1] * 0xff + raw[2];
     if (contentLen < (exponentSize + 4)) {
       throw runtime_error(getName() + " invalid input size for the public key");
     }
