@@ -1,4 +1,7 @@
+#ifndef BOOST_TEST_DYN_LINK
 #define BOOST_TEST_DYN_LINK
+#endif
+
 #define BOOST_TEST_NO_MAIN
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -10,7 +13,7 @@
 
 using namespace boost;
 
-BOOST_AUTO_TEST_SUITE(base64_cc)
+BOOST_AUTO_TEST_SUITE(test_base64_cc)
 
 BOOST_AUTO_TEST_CASE(test_Base64_Roundtrip) {
   std::string before("Some Random String"), after;
