@@ -111,9 +111,12 @@ static void startNewTransaction()
     cout<<"BEGIN TRANSACTION;"<<endl;
 }
 
-static void emitDomain(const DNSName& domain, const vector<ComboAddress>* primaries = nullptr)
+static void emitDomain(const ZoneName& domain, const vector<ComboAddress>* primaries = nullptr)
 {
   string iDomain = domain.toStringRootDot();
+  if (::arg().mustDo("delete-matching-domains")) {
+    cout << "DELETE FROM domains WHERE name=" << toLower(sqlstr(iDomain)) << ";" << endl;
+  }
   if (!::arg().mustDo("secondary")) {
     cout<<"insert into domains (name,type) values ("<<toLower(sqlstr(iDomain))<<",'NATIVE');"<<endl;
   }
@@ -134,7 +137,7 @@ static void emitDomain(const DNSName& domain, const vector<ComboAddress>* primar
 }
 
 bool g_doJSONComments;
-static void emitRecord(const DNSName& zoneName, const DNSName &DNSqname, const string &qtype, const string &ocontent, int ttl, const string& comment="")
+static void emitRecord(const ZoneName& zoneName, const DNSName &DNSqname, const string &qtype, const string &ocontent, int ttl, const string& comment="")
 {
   string qname = DNSqname.toStringRootDot();
   string zname = zoneName.toStringRootDot();
@@ -209,6 +212,7 @@ try
     ::arg().setSwitch("transactions","If target SQL supports it, use transactions")="no";
     ::arg().setSwitch("on-error-resume-next","Continue after errors")="no";
     ::arg().setSwitch("filter-duplicate-soa","Filter second SOA in zone")="yes";
+    ::arg().setSwitch("delete-matching-domains", "Delete existing matching domains before emitting new domain")="no";
     ::arg().set("zone","Zonefile to parse")="";
     ::arg().set("zone-name","Specify an $ORIGIN in case it is not present")="";
     ::arg().set("named-conf","Bind 8/9 named.conf to parse")="";
@@ -331,9 +335,9 @@ try
       cerr<<"\r100% done\033\133\113"<<endl;
     }
     else {
-      DNSName zonename;
+      ZoneName zonename;
       if(!::arg()["zone-name"].empty())
-        zonename = DNSName(::arg()["zone-name"]);
+        zonename = ZoneName(::arg()["zone-name"]);
 
       ZoneParserTNG zpt(zonefile, zonename);
       zpt.setMaxGenerateSteps(::arg().asNum("max-generate-steps"));
